#ifndef __MATRIXSSL3_H
#define __MATRIXSSL3_H

/* MatrixSSL capabilities */
#define SHARED_SESSION_CACHE_ENABLED            1
#define STATELESS_TICKETS_ENABLED               2
#define DH_PARAMS_ENABLED                       4
#define ALPN_ENABLED                            8
#define SNI_ENABLED                             16
#define OCSP_STAPLES_ENABLED                    32
#define CERTIFICATE_TRANSPARENCY_ENABLED        64

#endif
